--[[

$Revision: 53 $

(C) Copyright 2009 Bethink (bethink at naef dot com)
See LICENSE.txt for license terms.

]]


----------------------------------------------------------------------
-- Initialization

BossNotesAbilities = LibStub("AceAddon-3.0"):NewAddon("BossNotesAbilities",
	"AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("BossNotes")


----------------------------------------------------------------------
-- Constants

-- Database version
BOSS_NOTES_ABILITIES_DATABASE_VERSION = 1

-- Provider name
BOSS_NOTES_ABILITIES_PROVIDER = "ABILITIES"

-- Difficulties
BOSS_NOTES_ABILITIES_DIFFICULTY_NORMAL = 1
BOSS_NOTES_ABILITIES_DIFFICULTY_HEROIC = 2

-- DB defaults
local DB_DEFAULTS = {
	global = {
		npcs = { }
	}
}


	
----------------------------------------------------------------------
-- Core handlers

-- One-time initialization
function BossNotesAbilities:OnInitialize ()
	-- Database
	self.db = LibStub("AceDB-3.0"):New("BossNotesAbilitiesDB")
	self.db:RegisterDefaults(DB_DEFAULTS)
	self.db.global.databaseVersion = BOSS_NOTES_DATABASE_VERSION
end

-- Handles the (re-)enabling of this add-on
function BossNotesAbilities:OnEnable ()
	-- Register events
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	
	-- Register provider
	BossNotes:RegisterProvider(BOSS_NOTES_ABILITIES_PROVIDER, {
		OnSources = function (instance, encounter) return self:OnSources(instance, encounter) end
	})
end

-- Handles the disabling of this add-on
function BossNotesAbilities:OnDisable ()
	-- Unregister provider
	BossNotes:UnregisterProvider(BOSS_NOTES_ABILITIES_PROVIDER)
end


----------------------------------------------------------------------
-- Event handlers

-- Returns the abilities note
function BossNotesAbilities:OnSources (instance, encounter)
	-- Get NPC IDs
	local npcIds = (encounter and encounter.npcIds) or (instance and instance.npcIds)
	if not npcIds then
		return
	end
	
	-- Build HTML
	local html = { }
	table.insert(html, "<html><body>")
	for _, npcId in ipairs(npcIds) do
		local npc = self.db.global.npcs[npcId]
		if npc then
			-- Header
			table.insert(html, "<h1>" .. npc.name .. "</h1>")
			
			-- Acquire and sort spells
			local spells = { }
			for _, spell in pairs(npc.spells) do
				table.insert(spells, spell)
			end
			table.sort(spells, function (a, b)
				if a.difficulty ~= b.difficulty then
					return a.difficulty < b.difficulty
				end
				return a.name < b.name
			end)
			
			-- Append spells
			for _, spell in ipairs(spells) do
				local suffix = ""
				if bit.band(spell.difficulty, BOSS_NOTES_ABILITIES_DIFFICULTY_NORMAL) ~= 0 then
					suffix = suffix .. L["NORMAL"]
				end
				if bit.band(spell.difficulty, BOSS_NOTES_ABILITIES_DIFFICULTY_HEROIC) ~= 0 then
					if string.len(suffix) > 0 then
						suffix = suffix .. ", "
					end
					suffix = suffix .. L["HEROIC"]
				end
				table.insert(html, string.format("<p>" .. L["SPELL"] .. "</p>", spell.spellId, spell.name, suffix))
			end
			table.insert(html, "<br />")
		end
	end
	
	-- Placeholder
	if #html == 1 then
		table.insert(html, "<p>" .. L["ABILITIES_NOT_RECORDED"] .. "</p>")
	end
	
	-- Finish
	table.insert(html, "</body></html>")
	
	-- Return
	return {
		title = L["ABILITIES"],
		content = table.concat(html, "\n")
	}
end


----------------------------------------------------------------------
-- Events

-- Handles a combat log event
function BossNotesAbilities:COMBAT_LOG_EVENT_UNFILTERED (message, ...)
	-- Check for NPC
	local guid = select(3, ...)
	if string.sub(guid, 5, 5) ~= "3" then
		return
	end
	
	-- Check for boss
	local npcId = tonumber("0x" .. string.sub(guid, 9, 12))
	if not BossNotes:GetInstanceAndEncounter(npcId) then
		return
	end
	
	-- Check event
	local event = select(2, ...)
	if event ~= "SPELL_CAST_START" and event ~= "SPELL_CAST_SUCCESS" then
		return
	end

	-- Track spell
	local npcName = select(4, ...)
	local spellId = select(9, ...)
	local spellName = select(10, ...)
	self:TrackSpell(npcId, npcName, spellId, spellName)
end


----------------------------------------------------------------------
-- Support methods

-- Tracks a spell
function BossNotesAbilities:TrackSpell (npcId, npcName, spellId, spellName, difficulty)
	-- Track NPC
	local npc = self.db.global.npcs[npcId]
	if not npc then
		npc = {
			npcId = npcId,
			name = npcName,
			spells = { }
		}
		self.db.global.npcs[npcId] = npc
	end
	
	-- Track spell
	local spell = npc.spells[spellId]
	if not spell then
		spell = {
			spellId = spellId,
			name = spellName,
			difficulty = 0
		}
		npc.spells[spellId] = spell
	end
	
	-- Track difficulty
	local difficulty = bit.lshift(1, GetCurrentDungeonDifficulty() - 1)
	if bit.band(spell.difficulty, difficulty) == 0 then
		spell.difficulty = bit.bor(spell.difficulty, difficulty)
		
		-- Invalidate
		local instance, encounter = BossNotes:GetInstanceAndEncounter(npcId)
		local key = BossNotes:GetContextKey(instance, encounter)
		BossNotes:InvalidateContextKey(key)
	end
end
